/*
 * Copyright (c) 2014-2015 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *					Anthony Mallet on Tue Feb 18 2014
 */
#include "acoptitrack.h"

#include <sys/time.h>
#include <stdio.h>

#include "optitrack_c_types.h"
#include "codels.h"


/* --- Task cmd --------------------------------------------------------- */


/* --- Activity ping ---------------------------------------------------- */

/** Codel opti_ping_start of activity ping.
 *
 * Triggered by optitrack_start.
 * Yields to optitrack_send.
 * Throws optitrack_e_connection, optitrack_e_timeout,
 *        optitrack_e_network.
 */
genom_event
opti_ping_start(const optitrack_server_s *server,
                const optitrack_ids_ping_ctx *ping, int32_t *id,
                double *ts, int32_t *retries, genom_context self)
{
  struct timeval tv;
  if (!server || !server->natnet) return optitrack_e_connection(self);

  gettimeofday(&tv, NULL);
  *ts = tv.tv_sec + 1e-6 * tv.tv_usec;
  *id = ping->id + 1;
  *retries = 0;
  return optitrack_send;
}

/** Codel opti_ping_send of activity ping.
 *
 * Triggered by optitrack_send.
 * Yields to optitrack_pause_sleep.
 * Throws optitrack_e_connection, optitrack_e_timeout,
 *        optitrack_e_network.
 */
genom_event
opti_ping_send(const optitrack_server_s *server, genom_context self)
{
  optitrack_e_network_detail d;
  if (natnet_snd_ping(server->natnet, d.what, sizeof(d.what)))
    return optitrack_e_network(&d, self);
  return optitrack_pause_sleep;
}

/** Codel opti_ping_sleep of activity ping.
 *
 * Triggered by optitrack_sleep.
 * Yields to optitrack_send, optitrack_stop.
 * Throws optitrack_e_connection, optitrack_e_timeout,
 *        optitrack_e_network.
 */
genom_event
opti_ping_sleep(const optitrack_ids_ping_ctx *ping, int32_t id,
                int32_t *retries, genom_context self)
{
  if (ping->id < id) {
    if (++(*retries) < 7) return optitrack_send;
    return optitrack_e_timeout(self);
  }
  return optitrack_stop;
}

/** Codel opti_ping_stop of activity ping.
 *
 * Triggered by optitrack_stop.
 * Yields to optitrack_ether.
 * Throws optitrack_e_connection, optitrack_e_timeout,
 *        optitrack_e_network.
 */
genom_event
opti_ping_stop(const optitrack_ids_ping_ctx *ping, double ts,
               double *rtt, char info[64], genom_context self)
{
  struct timeval tv;

  gettimeofday(&tv, NULL);

  *rtt = tv.tv_sec + 1e-6 * tv.tv_usec - ts;
  snprintf(info, 64/*sizeof(info), grr C arrays ...*/, "%s", ping->response);
  return optitrack_ether;
}
